import os
import subprocess

from edi_core.util.file import dir_exists, get_file_name_without_extension


def extract_audio(file_path: str, output_path: str):
    result = subprocess.run(
        ['ffmpeg', '-i', file_path, '-q:a', '0', '-map', 'a', output_path],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE
    )
    if result.returncode != 0:
        raise RuntimeError(f"Failed to extract audio: {result.stderr.decode()}")


def split_audio(source_file: str, dest_dir: str, seconds: int):
    if not os.path.isfile(source_file):
        raise ValueError(f"Source file does not exist: {source_file}")
    if not dir_exists(dest_dir):
        os.makedirs(dest_dir, exist_ok=True)

    base_name = get_file_name_without_extension(source_file)
    segment_pattern = os.path.join(dest_dir, f"{base_name}_%03d.mp3")

    result = subprocess.run(
        ['ffmpeg', '-i', source_file, '-f', 'segment', '-segment_time', str(seconds), '-c', 'copy', segment_pattern],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE
    )

    if result.returncode != 0:
        raise RuntimeError(f"Failed to split audio: {result.stderr.decode()}")

    return sorted([os.path.join(dest_dir, f) for f in os.listdir(dest_dir) if f.startswith(base_name)],
                  key=lambda x: x.lower())



import winreg

SUB_KEY_KEYBOARD_LAYOUT = "SYSTEM\CurrentControlSet\Control\Keyboard Layout"

type_dict = {
    winreg.REG_BINARY: "Binary",
    winreg.REG_SZ: "String",
    winreg.REG_EXPAND_SZ: "Expandable String",
    winreg.REG_DWORD: "DWORD (32-bit number)",
    winreg.REG_MULTI_SZ: "Multi-String",
    winreg.REG_QWORD: "QWORD (64-bit number)",
}


def read_key_for_current_user(sub_key: str, name: str):
    hkey = winreg.OpenKey(winreg.HKEY_CURRENT_USER, sub_key, 0, winreg.KEY_READ)
    return winreg.QueryValueEx(hkey, name)


def get_keyboard_scancode_map():
    hkey = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, SUB_KEY_KEYBOARD_LAYOUT, 0, winreg.KEY_READ)
    value, type_id = winreg.QueryValueEx(hkey, "Scancode Map")
    print(type_id)
    print(value[0])



import os.path
import shutil
from io import TextIOWrapper

from edi_core.util.env import get_env


def dir_exists(path: str) -> bool:
    return os.path.isdir(path)


def file_exists(path: str) -> bool:
    return os.path.isfile(path)


def path_exists(path: str) -> bool:
    return os.path.exists(path)


def check_file_exists(path: str):
    if not file_exists(path):
        raise FileNotFoundError(f"The file at {path} does not exist.")


def move(src, dest):
    shutil.move(src, dest)


def delete_file(path: str):
    if file_exists(path):
        os.remove(path)


def get_file_name_without_extension(path: str) -> str:
    base_name = os.path.basename(path)
    name_without_extension, _ = os.path.splitext(base_name)
    return name_without_extension


def get_file_name_with_extension(path: str) -> str:
    return os.path.basename(path)


def get_output_dir():
    return get_env("OUTPUT_DIR")


def get_input_dir():
    return get_env("INPUT_DIR")


def to_output_path(relative_path: str):
    return os.path.join(get_output_dir(), relative_path)


def to_input_path(relative_path: str):
    return os.path.join(get_input_dir(), relative_path)


def ensure_parent_dir_exists(path: str):
    os.makedirs(os.path.dirname(path), exist_ok=True)


def save_to_output_dir(relative_path: str, content: str) -> str:
    output_path = to_output_path(relative_path)
    ensure_parent_dir_exists(output_path)
    with open(output_path, 'w', encoding='utf-8') as file:
        file.write(content)
    return output_path


def open_file_in_output(relative_path: str) -> TextIOWrapper:
    output_path = to_output_path(relative_path)
    ensure_parent_dir_exists(output_path)
    return open(output_path)


def open_file_in_input(relative_path: str) -> TextIOWrapper:
    input_path = to_input_path(relative_path)
    return open(input_path)


def read_file(path: str) -> str:
    check_file_exists(path)
    with open(path, 'r', encoding='utf-8') as file:
        return file.read()



import os
import platform

from dotenv import load_dotenv

dotenv_loaded = False


def load_env():
    global dotenv_loaded
    if not dotenv_loaded:
        load_dotenv()
        dotenv_loaded = True


def get_env(key: str, default: str = None):
    load_env()
    value = os.environ.get(key)
    if value:
        return value

    if default is None:
        raise ValueError('env key not found')
    else:
        return default


def get_platform():
    return platform.platform()


def get_system():
    return platform.system()


def is_windows():
    return get_system() == 'Windows'



from abc import ABC

import alibabacloud_oss_v2 as oss
from alibabacloud_oss_v2 import GetObjectMetaRequest, GetObjectMetaResult, GetObjectRequest
from loguru import logger as log


class OssClient(ABC):
    def get_object_metadata(self, key: str):
        pass

    def get_object(self, key: str, target_path: str):
        pass

    def put_object(self, source_path: str, key: str):
        pass

    def delete_object(self, key: str):
        pass


class AliCloudOssClient(OssClient):
    _config: oss.Config
    _client: oss.Client
    _bucket: str

    def __init__(self, region: str, access_key: str, secret_key: str, endpoint: str, bucket: str):
        config = oss.config.load_default()
        config.region = region
        config.credentials_provider = oss.credentials.StaticCredentialsProvider(access_key_id=access_key,
                                                                                access_key_secret=secret_key)
        config.endpoint = endpoint
        self._config = config
        self.print_config()
        self._client = oss.Client(self._config)
        self._bucket = bucket

    def get_config(self) -> oss.Config:
        return self._config

    def get_object_metadata(self, key: str) -> GetObjectMetaResult:
        request = GetObjectMetaRequest(bucket=self._bucket, key=key)
        return self._client.get_object_meta(request)

    def get_object(self, key: str, target_path: str):
        log.debug(f"downloading {key} to {target_path}")
        request = GetObjectRequest(bucket=self._bucket, key=key)
        result = self._client.get_object(request)
        with result.body as body_stream:
            data = body_stream.read()
            with open(target_path, 'wb') as f:
                f.write(data)
        log.debug(f"finished downloading {key} to {target_path}")

    def put_object(self, source_path: str, key: str):
        pass

    def delete_object(self, key: str):
        pass

    def print_config(self):
        config_attrs = vars(self._config)
        config_str = "\n".join(f"{key}={value}" for key, value in config_attrs.items())
        log.debug(f"OSS Config: {config_str}")
